﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.DTO;
using WingInterfaceLibrary.LiveConsultation.Consultation;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.LiveConsultation.QualityControl;
using WingInterfaceLibrary.DTO.Patient;
using WingInterfaceLibrary.OpLog;

namespace WingInterfaceLibrary.LiveConsultation
{
    /// <summary>
    /// 预约会诊接口
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>  
    /// <docSecondDirectory>实时会诊服务接口</docSecondDirectory>   
    /// <docThirdDirectory>预约</docThirdDirectory>  
    public interface ILiveConsultationService
    {
        /// <summary>
        /// 查询上级机构集合
        /// </summary>
        /// <param name="request">查询上级机构集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationBaseDTO>> FindParentOrganizationsAsync(FindHigherOrganizationsRequest request);

        /// <summary>
        /// 查询基层机构集合
        /// </summary>
        /// <param name="request">查询基层机构集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationBaseDTO>> FindGrassRootsOrganizationsAsync(FindGrassRootsOrganizationsRequest request);

        /// <summary>
        /// 查询机构专家集合
        /// </summary>
        /// <param name="request">查询机构专家集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationExpertDTO>> FindOrganizationExpertsAsync(FindOrganizationExpertsRequest request);

        /// <summary>
        /// 申请会诊
        /// </summary>
        /// <param name="request">申请会诊请求实体</param>
        /// <returns>会诊编码</returns>
        /// <value>ConsultationCodeXXXX</value>
        /// <errorCodes>UserNotFind,PatientDataIsEmpty,PatientNameIsEmpty,PatientPhoneIsEmpty,DeviceCodeIsEmpty,ConsultationTimeGreaterThanCurrentTime,ExpertOrganizationCodeIsEmpty,ConsultationExpertNotExist,ApplyUserCodeIsEmpty,ConsultationApplyUserIllegal</errorCodes>
        Task<string> ApplyConsultationAsync(ApplyConsultationRequest request);        

        /// <summary>
        /// 修改会诊信息
        /// </summary>
        /// <param name="request">申请会诊请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes></errorCodes>
        Task<bool> UpdateConsultationAsync(UpdateConsultationRequest request);       

        /// <summary>查询助理服务专家集合</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<OrganizationExpertDTO>> FindAssistantExpertsAsync(FindAssistantExpertsRequest request);

        /// <summary>
        /// 分页查询会诊记录
        /// </summary>
        /// <param name="request">分页查询会诊记录请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<PageResult<ConsultationDetailDTO>> FindConsultationsByPageAsync(FindConsultationByPageRequest request);

        /// <summary>
        /// 查询会诊详情
        /// </summary>
        /// <param name="request">查询会诊详情请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<ConsultationDetailDTO> FindConsultationDetailAsync(FindConsultationDetailRequest request);

        /// <summary>
        /// 批准会诊申请
        /// </summary>
        /// <param name="request">批准会诊申请请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>UserNotFind,ConsultationTimeGreaterThanCurrentTime,ConsultationExpertNotExist,ConsultationNorightError,ConsultationStatusError</errorCodes>
        Task<bool> ApprovalConsultationAsync(ApprovalConsultationRequest request);

        /// <summary>
        /// 拒绝申请的会诊请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> RejectApplyConsultationAsync(RejectApplyConsultationRequest request);

        /// <summary>
        /// 撤回会诊申请
        /// </summary>
        /// <param name="request">撤回会诊申请请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> RevokeConsultationAsync(RevokeConsultationRequest request);

        /// <summary>
        /// 删除会诊记录
        /// </summary>
        /// <param name="request">删除会诊记录请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> DeleteConsultationAsync(DeleteConsultationRequest request);

        /// <summary>
        /// 保存预约会诊图像
        /// 累加保存
        /// </summary>
        /// <param name="request">保存预约会诊图像请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <show>false</show>
        Task<bool> UpdateConsultationFilesInfoAsync(UpdateConsultationFilesInfoRequest request);

        /// <summary>
        /// 查询自己所在的机构专家助理集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<ConsultationAssistantDTO>>  FindOrganizationAssistantsAsync(FindOrganizationAssistantsRequest request);

        /// <summary>
        /// 更新设置自己的专家助理
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>NoExpertFeature,AssistantNotExisted,NotExpertAssistant</errorCodes>
        Task<bool> UpdateMyOrganizationAssistantAsync(UpdateMyOrganizationAssistantRequest request);

        /// <summary>
        ///查询已创建已登记的病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ClientPatientInfoBaseDTO>> FindConsultationPatientAsync(FindConsultationPatientPageRequest request);

        /// <summary>
        /// Initiate a diagnosis meeting
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        /// <errorCodes>ConsultationNotExisted,ConsultationExpertOrgError,ConsultationStatusError,ConsultationNotOneOnLine,ConsultationDBError</errorCodes>
        Task<InitiateLiveConsultationResult> InitiateLiveConsultationAsync(InitiateLiveConsultationRequest request);

        /// <summary>
        ///  Initiated   other Users or devices in  diagnosis meeting
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<InviteInLiveConsultationResult> InviteInLiveConsultationAsync(InviteInLiveConsultationRequest request);

        /// <summary>
        ///  Cancel initiating  other Users or devices in  diagnosis meeting
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<CancelInvitingInLiveConsultationResult> CancelInvitingInLiveConsultationAsync(CancelInvitingInLiveConsultationRequest request);

        /// <summary>
        /// Cancel the initiating a diagnosis meeting
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<CancelLiveConsultationResult> CancelLiveConsultationAsync(CancelLiveConsultationRequest request);

        /// <summary>
        /// Accept a diagnosis request
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<AcceptLiveConsultationResult> AcceptLiveConsultationAsync(AcceptLiveConsultationRequest request);

        /// <summary>
        /// Reject a diagnosis request
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        Task<RejectLiveConsultationResult> RejectLiveConsultationAsync(RejectLiveConsultationRequest request);
        /// <summary>
        /// Join a diagnosis request
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<JoinLiveConsultationResult> JoinLiveConsultationAsync(JoinLiveConsultationRequest request);
        /// <summary>
        /// Leave a diagnosis request
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<LeaveLiveConsultationResult> LeaveLiveConsultationAsync(LeaveLiveConsultationRequest request);
        /// <summary>
        /// 设置开启或关闭静音状态
        /// </summary>
        /// <param name="request">设置开启或关闭静音状态的请求</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<MuteLiveConsultationResult> MuteLiveConsultationAsync(MuteLiveConsultationRequest request);
        /// <summary>
        /// 开启或关闭视频
        /// </summary>
        /// <param name="request">开启或关闭视频请求</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<SwitchLiveConsultationVideoResult> SwitchLiveConsultationVideoAsync(SwitchLiveConsultationVideoRequest request);
        /// <summary>
        /// Send HeartRate request in a diagnosis metting
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<LiveConsultationHeartRateResult> HeartRateAsync(LiveConsultationHeartRateRequest request);

        /// <summary>
        /// 发送互动白板操作数据
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> SendInteractiveBoardDataAsync(SendInteractiveBoardDataRequest request);

        /// <summary>
        /// 添加随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> AddFollowUpVisitInfoAsync(AddFollowUpVisitInfoRequest request);

        /// <summary>
        /// 删除随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> DeleteFollowUpVisitInfoAsync(DeleteFollowUpVisitInfoRequest request);

        /// <summary>
        /// 修改随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> UpdateFollowUpVisitInfoAsync(UpdateFollowUpVisitInfoRequest request);

        /// <summary>
        /// 查询随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<List<FollowUpVisitDTO>> GetFollowUpVisitInfoAsync(GetFollowUpVisitInfoRequest request);

        /// <summary>
        /// 查询随访信息详情
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,FollowUpVisitInfoNotFound</errorCodes>
        /// <show>false</show>
        Task<FollowUpVisitDTO> GetFollowUpVisitInfoDetailAsync(GetFollowUpVisitInfoDetailRequest request);

        /// <summary>
        /// 设置需要随访
        /// </summary>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> SetNeedFollowUpVisitAsync(SetNeedFollowUpVisitRequest request);

        /// <summary>
        /// 新增会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> AddConsultationEvaluateAsync(AddConsultationEvaluateRequest request);

        /// <summary>
        /// 删除会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> DeleteConsultationEvaluateAsync(DeleteConsultationEvaluateRequest request);

        /// <summary>
        /// 修改会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> UpdateConsultationEvaluateAsync(UpdateConsultationEvaluateRequest request);

        /// <summary>
        /// 查询会诊评价详情
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,ConsultationEvaluateNotFound</errorCodes>
        /// <show>false</show>
        Task<ConsultationEvaluateDTO> GetConsultationEvaluateDetailAsync(GetConsultationEvaluateRequest request);

        /// <summary>
        /// 查询扫查部位列表 
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        Task<List<string>> FindScanPositionsAsync(TokenRequest request);

        /// <summary>
        /// 接受会诊邀请
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> AcceptInvitationAsync(AcceptInvitationRequest request);

        /// <summary>
        /// 服务器间会诊通知
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> SyncServerMessageAsync(SyncReceiveServiceDataRequest request);
    }
}